# -*- coding: utf-8 -*-

from django.forms.models import ModelForm
from django import forms
from django.contrib.auth.forms import PasswordResetForm, SetPasswordForm
from django.contrib.auth.forms import UserCreationForm
from django.utils.translation import ugettext_lazy as _

from bookLoan.models import Reader
from django.contrib.auth.models import User
        
class MinimalReaderForm(UserCreationForm):
    
    first_name = forms.CharField(label="Primeiro nome", max_length=30, required=True)
    last_name = forms.CharField(label="Sobrenome", max_length=30, required=True)
    #TODO: set initial value to 'escolha um gênero'. It must rewrite __init__ form
    gender = forms.ChoiceField(label="Gênero", widget = forms.Select(), choices=Reader.GENDER_CHOICES, required=True)
    email = forms.EmailField(label="Email", required=True)
    
    # email must be unique
    def clean_email(self):
        try:
            User.objects.get(email=self.cleaned_data['email'])
            raise forms.ValidationError(u'Email já cadastrado. Escolha outro.')
        except User.DoesNotExist:
            return self.cleaned_data['email']
    
    def save(self, commit=True):
        user = super(MinimalReaderForm, self).save(commit=False)
        user.first_name = self.cleaned_data["first_name"]
        user.last_name = self.cleaned_data["last_name"]
        user.email = self.cleaned_data["email"]
        user.is_active = False
        user.save()
            
        reader = Reader.objects.create(user=user)
        reader.gender = self.cleaned_data["gender"]
        if commit:
            reader.save()
        return reader
    
    def __init__(self, *args, **kwargs):
        super(MinimalReaderForm, self).__init__(*args, **kwargs)
        self.fields.keyOrder = ['first_name', 'last_name', 'gender', 'email', 'username', 'password1', 'password2']
    
class CompleteReaderForm(ModelForm):
    class Meta:
        model = Reader
        fields = ('gender','photo','birth_date','ocupation')
        widgets = {
                   #TODO: put this-> input_formats=['%d/%m/%Y']
                   'birth_date': forms.DateInput(format="%d/%m/%Y"),
        }
        
    def clean_gender(self):
        #turn first_name required in form, not DB
        if self.cleaned_data['gender']:
            return self.cleaned_data['gender']
        raise forms.ValidationError('Este campo é obrigatório.')

class BiglibPasswordResetForm(PasswordResetForm):
    email = forms.EmailField(label="Email", max_length=75)
    
class BiglibSetPasswordForm(SetPasswordForm):
    """
    A form that lets a user change set his/her password without
    entering the old password
    """
    new_password1 = forms.CharField(label="Nova senha", widget=forms.PasswordInput(attrs={'placeholder':u'senha'}))
    new_password2 = forms.CharField(label=u"Confirmação da nova senha", widget=forms.PasswordInput(attrs={'placeholder':u'confirmação de senha'}))

class BiglibPasswordChangeForm(BiglibSetPasswordForm):
    """
    A form that lets a user change his/her password by entering
    their old password.
    """
    old_password = forms.CharField(label=u"Senha atual", widget=forms.PasswordInput(attrs={'placeholder':'senha atual'}))

    def clean_old_password(self):
        old_password = self.cleaned_data["old_password"]
        if not self.user.check_password(old_password):
            raise forms.ValidationError("Voce errou ao digitar a sua senha atual. Digite-a novamente.")
        return old_password
    
class ChangeEmailForm(forms.Form):
    email = forms.EmailField(label="Novo email", max_length=75)
 
    # email must be unique
    def clean_email(self):
        try:
            User.objects.get(email=self.cleaned_data['email'])
            raise forms.ValidationError(u'Email já cadastrado. Escolha outro.')
        except User.DoesNotExist:
            return self.cleaned_data['email']
              
