#-*- coding: utf-8 -*-

from django.shortcuts import render_to_response, redirect
from django.template.context import RequestContext
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib.auth.decorators import login_required
from django.contrib.auth import logout
from django.contrib.sites.models import Site
from django.views.decorators.csrf import csrf_protect
from django.views.decorators.cache import never_cache
from smtplib import SMTPException
from django.core.urlresolvers import reverse
import re
from datetime import datetime
from django.contrib.auth.forms import AuthenticationForm

from bookLoan.models import Activation, EmailChangeAuth
from accounts.forms import CompleteReaderForm, ChangeEmailForm
from accounts.forms import MinimalReaderForm
from bookLoan.utils import send_activation_email, send_activation_changed_email,\
    domain_suffix


@login_required        
def edit_profile(request, username, template_name):
    myself = request.user.get_profile()
    if request.method == 'POST':
        next_url=reverse('profile', kwargs={'username':myself.user.username})
        rform = CompleteReaderForm(request.POST,request.FILES,instance=myself)
        if rform.is_valid():  
            rform.save()
            #perfil alterado salvo
            return HttpResponseRedirect(next_url)
    else:
        rform = CompleteReaderForm(instance=myself)            
    return render_to_response(template_name, {'form': rform},
                              context_instance=RequestContext(request))
@csrf_protect
@never_cache
def welcome(request):
    if request.user.is_authenticated():
        return redirect(to='bookLoan.views.profile', username=request.user.username)
    elif request.method == 'POST':
        # new reader to register
        rform = MinimalReaderForm(data=request.POST)
        if rform.is_valid():
            reader = rform.save()
            try:
                send_activation_email(reader.user, request)
            except SMTPException:
                #TODO: put a more elegant default error message
                return HttpResponse(u'Não foi possível enviar email de confirmação')
            return render_to_response('accounts/registration_complete.html', context_instance=RequestContext(request))
    else:
        rform = MinimalReaderForm()
        
    lform = AuthenticationForm()
    
    return render_to_response('accounts/welcome.html', {'domain_suffix': domain_suffix(), 'rform': rform, 'lform': lform},
                              context_instance=RequestContext(request))

def activate(request, key):
    try:
        akey = Activation.objects.get(key=key)
        user = akey.user
        user.is_active = True
        user.save()
        akey.delete()
        return render_to_response('accounts/activation_complete.html', {}, RequestContext(request))
    except Activation.DoesNotExist:
        return render_to_response("error.html", {}, RequestContext(request))
    
def changed_email_activate(request, key):
    try:
        akey = EmailChangeAuth.objects.get(auth_key=key)
        
        if datetime.now() > akey.expire_date:
            return render_to_response("error.html",
                                      {'message': u'A chave de ativação de mudança de email expirou. Faça uma nova requisição.'},
                                      RequestContext(request))
        user = akey.user
        user.email = akey.new_email
        user.save()
        akey.delete()
        
        return render_to_response('accounts/email_changed_done.html',
                                  context_instance=RequestContext(request))
    except EmailChangeAuth.DoesNotExist:
        return render_to_response("error.html",
                                  {'message': u'URL inválida. Provavelmente este link já foi utilizado.'},
                                  RequestContext(request))

def logged(request):
    next_url = '../%s' % request.user.username
    return HttpResponseRedirect(next_url)

def biglib_logout(request):
    logout(request)
    return render_to_response('accounts/logout.html',
                              context_instance=RequestContext(request))


@csrf_protect
@never_cache
@login_required
def change_email(request, template_name):
    if request.method == 'POST':
        cemail_form = ChangeEmailForm(data=request.POST)
        if cemail_form.is_valid():           
            new_email = cemail_form.cleaned_data['email']            
            try:               
                send_activation_changed_email(request,request.user,new_email)
            except SMTPException:
                #TODO: put a more elegant default error message
                return HttpResponse(u'Não foi possível enviar email de confirmação')
            return render_to_response('accounts/email_change_complete.html',
                                      context_instance=RequestContext(request))
    else:
        cemail_form = ChangeEmailForm()
        
    return render_to_response(template_name, {'form': cemail_form},
                              RequestContext(request))
    
def register_reader(request):
    if request.user.is_authenticated():
        logout(request)
    if request.method == 'POST':
        rform = MinimalReaderForm(request.POST)
        if rform.is_valid():
            reader = rform.save()
            try:
                send_activation_email(reader.user, request)
            except SMTPException:
                render_to_response('error.html', {}, RequestContext(request))
            return render_to_response('accounts/registration_complete.html',
                                      RequestContext(request))
    else:
        rform = MinimalReaderForm()
    return render_to_response('accounts/registration.html', {'domain_suffix': domain_suffix(), 'rform': rform},
                              RequestContext(request))
