# -*- coding: utf-8 -*-
from django.core.exceptions import ValidationError
import re

# wrapper to is_isbn_10 function.
def validate_isbn_10(isbn):
    if not is_isbn_10:
        raise ValidationError(u'ISBN-10 inválido')
    
# wrapper to is_isbn_13 function.
def validate_isbn_13(isbn):
    if not is_isbn_13:
        raise ValidationError(u'ISBN-13 inválido')
    
# checks the validity of an ISBN-10.
def is_isbn_10(isbn):
    if (len(isbn) != 10):
        return False
    chars = list(isbn)
    total = 0
    digit = 10
    for char in chars:
        if (char == 'X' or char == 'x'):
            char = "10"
        total += digit * int(char)
        digit -= 1
    remainder = total % 11
    if remainder == 0:
        return True
    else:
        return False


# checks the validity of an ISBN-13 number.
def is_isbn_13(isbn):
    if (len(isbn) != 13):
        return False
    chars = list(isbn)
    total = 0
    count = 0
    for i in range(12):
        if (count % 2 == 0):
            total += int(chars[i])
        else:
            total += 3 * int(chars[i])
        count += 1
    if int(chars[12]) == (10 - (total % 10)) % 10:
        return True
    else:
        return False

# takes an isbn and return only digits
# and possibly an 'X'
def isbn_clear(isbn):
    short=re.sub("\W","",isbn)
    return re.sub("\D","X",short)


# convert an valid ISBN-10 to ISBN-13
def from_i10_to_i13(isbn):
    if not is_isbn_10(isbn):
        raise "Invalid ISBN-10"
    stem = "978" + isbn[:-1]
    return stem + check_digit_i13(stem)

# convert an valid ISBN-13 to ISBN-10
def from_i13_to_i10(isbn):
    if not is_isbn_13(isbn):
        raise "Invalid ISBN-13"
    if isbn[:3] == "978":
        stem = isbn[3:-1]
        return stem + check_digit_i10(stem)
    else:
        raise "ISBN-13 not convertible"
    
# computes the ISBN-10 check digit
# based on the first 9 digits.
def check_digit_i10(stem):
    chars = list(stem)
    total = 0
    digit = 10
    for char in chars:
        total += digit * int(char)
        digit -= 1
    check = 11 - (total % 11)
    if check == 10:
        return "X"
    elif check == 11:
        return "0"
    else:
        return str(check)



# computes the ISBN-13 check digit
# based on the first 12 digits.    
def check_digit_i13(stem):
    chars = list(stem)
    total = 0
    count = 0
    for char in chars:
        if (count % 2 == 0):
            total += int(char)
        else:
            total += 3 * int(char)
        count+=1
    check = 10 - (total % 10)
    if check == 10:
        return "0"
    else:
        return str(check)