# -*- coding: utf-8 -*-

from django.template import loader
import random
from django.core.mail import send_mail
from django.template.context import RequestContext
from django.contrib.sites.models import Site
from smtplib import SMTPException
from django.core.urlresolvers import reverse
from datetime import date, timedelta
import Image
import urllib2
from django.core.files.base import ContentFile
from StringIO import StringIO
import re
from django.conf import settings

from bookLoan.models import Book, BookInstance
from tagging.models import Tag
from bookLoan.models import Activation, EmailChangeAuth

class InvalidRegistrationKeyException(Exception): pass

def generate_key():
    return ''.join([chr(random.randint(97, 122)) for _i in range(30)])

def activate_user(username, key):
    confirmation = Activation.objects.get(key=key)
    if str(confirmation.user.username) == str(username):
        confirmation.user.is_active = True
        confirmation.user.save()
        confirmation.delete() 
    else:
        raise InvalidRegistrationKeyException

def send_activation_email(user, request):
    key = generate_key()
    e = Activation(key=key, user=user)
    e.save()
    
    message = loader.render_to_string(template_name='accounts/activation_email.txt', 
                                      dictionary={'user': user, 'site': Site.objects.get_current(), 'url_to_key': reverse('account_activation', kwargs={'key':key})}, 
                                    context_instance=RequestContext(request))
    try:
        send_mail(subject=settings.EMAIL_SUBJECT_PREFIX + u'Ativação de conta', message=message, 
                  from_email=settings.EMAIL_HOST_USER, recipient_list=[user.email], 
                  fail_silently=False)
    except SMTPException:
        raise SMTPException
    
def activate_changed_email(username, key):
    confirmation = EmailChangeAuth.objects.get(key=key)
    if str(confirmation.user.username) == str(username):
        confirmation.user.is_active = True
        confirmation.user.save()
        confirmation.delete() 
    else:
        raise InvalidRegistrationKeyException
    
def send_activation_changed_email(request, user ,new_email):
    key = generate_key()
    expire_date = date.today() + timedelta(days=7)
    e = EmailChangeAuth(auth_key=key, user=user, new_email=new_email, expire_date=expire_date)
    e.save()

    message = loader.render_to_string(template_name='accounts/email_changed_email.txt', 
                                      dictionary={'user': user, 'site': Site.objects.get_current(), 'url_to_key': reverse('changed_email_activation', kwargs={'key':key})}, 
                                    context_instance=RequestContext(request))
    try:
        send_mail(subject=settings.EMAIL_SUBJECT_PREFIX + u'Alteração de email', message=message, 
                  from_email=settings.EMAIL_HOST_USER, recipient_list=[new_email], 
                  fail_silently=False)
    except SMTPException:
        raise SMTPException
    
def make_books_tags():
    allbooks = Book.objects.all()
    for book_to_be_tagged in allbooks:
        #instances = BookInstance.objects.filter(book=book_to_be_tagged)
        tags = Tag.objects.usage_for_model(BookInstance, counts=True, filters=dict(book=book_to_be_tagged))
        result = [(tag.name, tag.count) for tag in tags]
        #count = tags.count
        #result = [(item, count(item)) for item in set(tags)]
        result.sort(key=lambda x:-x[1])
        new_tags = [tag for tag, num in result if num >= 2][0:4]   #the two most appeared tags in books instances
        new_tags_str = ''
        for tag in new_tags:
            new_tags_str += ' "'+str(tag)+'"'
        book_to_be_tagged.set_tags(new_tags_str)
        
def send_book_request_email(request, book_loan):
    
    msg = loader.render_to_string(template_name='bookLoan/book_request_email.txt', 
                                      dictionary={'user': book_loan.book.owner.user,'site': Site.objects.get_current(),
                                                  'url_to_response': reverse('book_response',
                                                                             kwargs={'book_loan_id': book_loan.id}),
                                                  'book_loan': book_loan}, 
                                      context_instance=RequestContext(request))

    try:
        send_mail(subject=settings.EMAIL_SUBJECT_PREFIX + u'Pedido de empréstimo', message=msg, 
                  from_email=settings.EMAIL_HOST_USER, recipient_list=[book_loan.book.owner.user.email], 
                  fail_silently=False)
    except SMTPException:
        raise SMTPException    

def send_book_response_email(request, book_loan):
    
    msg = loader.render_to_string(template_name='bookLoan/book_response_email.txt', 
                                      dictionary={'user': book_loan.requester.user ,'site': Site.objects.get_current(),
                                                  'url_to_view': reverse('loan_detail',
                                                                             kwargs={'loan_id': book_loan.id}),
                                                  'book_loan': book_loan}, 
                                      context_instance=RequestContext(request))
    try:
        send_mail(subject=settings.EMAIL_SUBJECT_PREFIX + u'Resposta de empréstimo', message=msg, 
                  from_email=settings.EMAIL_HOST_USER, recipient_list=[book_loan.requester.user.email], 
                  fail_silently=False)
    except SMTPException:
        raise SMTPException
    
def send_book_returning_email(request, book_loan):
    
    msg = loader.render_to_string(template_name='bookLoan/book_returning_email.txt', 
                                      dictionary={'user': book_loan.book.owner.user,'site': Site.objects.get_current(),
                                                  'url_to_view': reverse('loan_detail',
                                                                             kwargs={'loan_id': book_loan.id}),
                                                  'book_loan': book_loan}, 
                                      context_instance=RequestContext(request))
    try:
        send_mail(subject=settings.EMAIL_SUBJECT_PREFIX + u'Devolução de livro', message=msg, 
                  from_email=settings.EMAIL_HOST_USER, recipient_list=[book_loan.book.owner.user.email], 
                  fail_silently=False)
    except SMTPException:
        raise SMTPException
    
def send_invite_email(request, email_addr, custom_msg):
    
    msg = loader.render_to_string(template_name='friendship/invite_email.txt', 
                                      dictionary={'site': Site.objects.get_current(),
                                                  'register': reverse('register'),
                                                  'user': request.user,
                                                  'custom_msg': custom_msg}, 
                                      context_instance=RequestContext(request))
    try:
        send_mail(subject=settings.EMAIL_SUBJECT_PREFIX + u'Convite', message=msg, 
                  from_email=settings.EMAIL_HOST_USER, recipient_list=[email_addr], 
                  fail_silently=False)
    except SMTPException:
        raise SMTPException    

def send_request_friend_email(user, friend, request):
    url_to_response = reverse('ask_friendship_confirm', kwargs={'username':friend.username})
    message = loader.render_to_string(template_name='friendship/request_friend_email.txt', 
                                      dictionary={'user': user, 'friend': friend,
                                                  'site': Site.objects.get_current(),
                                                  'url_to_response': url_to_response 
                                                  }, 
                                      context_instance=RequestContext(request))
    try:
        send_mail(subject=settings.EMAIL_SUBJECT_PREFIX + u'Requisição de amizade', message=message, 
                  from_email=settings.EMAIL_HOST_USER, recipient_list=[user.email], 
                  fail_silently=False)
    except SMTPException:
        raise SMTPException
    
def send_added_friend_email(user, friend, request):
    message = loader.render_to_string(template_name='friendship/added_friend_email.txt', 
                                      dictionary={'user': user, 'friend': friend,
                                                  'site': Site.objects.get_current(),
                                                  'friend_url': friend.get_profile().get_absolute_url() 
                                                  }, 
                                      context_instance=RequestContext(request))
    try:
        send_mail(subject=settings.EMAIL_SUBJECT_PREFIX + u'Requisição de amizade respondida', message=message, 
                  from_email=settings.EMAIL_HOST_USER, recipient_list=[user.email], 
                  fail_silently=False)
    except SMTPException:
        raise SMTPException

def download_image(name, image, url):
    input_file=StringIO(urllib2.urlopen(url).read())
    output_file=StringIO()
    img=Image.open(input_file)
    if img.mode != "RGB":
        img=img.convert("RGB")
    img.save(output_file,"JPEG")
    image.save(name+".jpg", ContentFile(output_file.getvalue()), save=False)

def domain_suffix():    
    domain_name = Site.objects.get_current().domain
    return re.sub(r'^(http://)?(www.)?', '', domain_name)

