#-*- coding: utf-8 -*-
from django.shortcuts import render_to_response, get_object_or_404
from django.template.context import RequestContext
from django.http import HttpResponse, HttpResponseRedirect, Http404,\
    HttpResponseForbidden, HttpResponseBadRequest
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.contrib.auth import logout
from smtplib import SMTPException
from django.utils import simplejson
from django.core.urlresolvers import reverse
from datetime import date, timedelta

from biblio.webquery import xisbn
from bookLoan.models import BookInstance, Author, Book,\
    BookLoan, Publisher, Reader
from bookLoan.forms import RequestBookForm ,BookForm, SearchBookForm, EditBookInstanceForm
from bookLoan.utils import send_book_request_email, send_book_response_email,\
    send_book_returning_email
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage,\
    InvalidPage
from tagging.models import Tag, TaggedItem
from tagging.utils import calculate_cloud
from tagging.utils import LOGARITHMIC as dist_LOG
from bookLoan.utilities.isbn import is_isbn_10, is_isbn_13, isbn_clear
from django.views.decorators.csrf import csrf_exempt



@login_required
def my_tags(request, username):
    user = request.user
    if user.username != username:
        return HttpResponseForbidden(u'Nao é possivel acessar tags de outro usuario')
    my_tags=Tag.objects.usage_for_model(BookInstance, counts=True, filters=dict(owner__in=request.user.get_profile().get_friends_ids_and_mine()))
    calculate_cloud(my_tags, steps=6, distribution=dist_LOG)
    return render_to_response("bookLoan/my_cloud_tags.html", {'my_tags': my_tags}, context_instance=RequestContext(request)) 

@login_required
def my_books_tag(request, username, slug): 
    user = request.user
    if user.username != username:
        return HttpResponseForbidden(u'Nao é possivel acessar tags de outro usuario')
    
    query_tag = Tag.objects.get(slug=slug)    
    list_obj=TaggedItem.objects.get_by_model(BookInstance, query_tag)
    entries=list_obj.filter(owner__in=user.get_profile().get_friends_ids_and_mine())
    entries = entries.order_by('joined_date')

    return render_to_response("bookLoan/my_books_with_tag.html", dict(tag=slug, entries=entries), context_instance=RequestContext(request)) 

@login_required
def search_book(request, limit=10):
    if request.method == 'GET':
        # gives an set of books that reader may have
        #TODO: get more meaningful books.
        book_list = Book.objects.all()[:limit]
        search_form = SearchBookForm()
    elif request.method == 'POST':
        # handle an user search
        search_form = SearchBookForm(data=request.POST)
        if search_form.is_valid():
            what = search_form.cleaned_data['what']
            search = search_form.cleaned_data['search']
            if what == SearchBookForm.ISBN:
                if len(what) == 10:
                    try:
                        book = Book.objects.get(isbn10=search)
                    except Book.DoesNotExist:
                        book_list = []
                    else:
                        book_list = [book]
                else:
                    try:
                        book = Book.objects.get(isbn13=search)
                    except Book.DoesNotExist:
                        book_list = []
                    else:
                        book_list = [book]
            elif  what == SearchBookForm.TITLE:
                book_list = Book.objects.filter(title__icontains=search)[:limit]
                if len(book_list) == 0:
                    book_list = []
            elif  what == SearchBookForm.AUTHOR:
                book_list = Book.objects.filter(authors__name__icontains=search)[:limit]
                if len(book_list) == 0:
                    book_list = []
        else:
            book_list = []
    return render_to_response('bookLoan/search_book.html', {'sform': search_form, 'book_list': book_list},\
                              RequestContext(request))

@login_required
def register_book(request):
    msg = ''
    
    if request.method == 'POST' and 'auto_fill' in request.POST:
        isbn = isbn_clear(request.POST['isbn'])
        query = xisbn.XisbnQuery()
        if is_isbn_10(isbn) or is_isbn_13(isbn):
            book = query.query_bibdata_by_isbn(isbn)
            if book:
                bform = BookForm(initial={'isbn': book[0].id, 'title': book[0].title,\
                                      'author': book[0].authors[0],\
                                      'publisher': book[0].publisher})
            else:
                bform = BookForm(initial={'isbn':request.POST['isbn']})
                msg = 'a'
        else:
            #TODO: corrigir isso
            bform = BookForm(initial={'isbn':request.POST['isbn']})
            msg = 'a'
    
    elif request.method == 'POST':
        bform = BookForm(request.POST, request.FILES)
        if bform.is_valid():
            book = bform.save(commit=False)
            book.registered_by = request.user.get_profile()
            
            try:
                book.save()
            except Book.DoesNotExist:
                return HttpResponse(u'Não conseguimos cadastrar o livro')
            
            # (if needed) create extra authors and link them to book
            number_of_authors = request.POST['total_author']
            for i in range(1, int(number_of_authors) + 1):
                if i == 1:
                    author_name = request.POST['author']
                else:
                    author_name = request.POST['author_' + str(i)]
                try:
                    author = Author.objects.get(name=author_name)
                except Author.DoesNotExist:
                    author = Author(name=author_name)
                    author.save()
                book.authors.add(author)
                
            binstance = BookInstance()
            binstance.owner = request.user.get_profile()
            try:
                book.save()
                binstance.book = book
                binstance.save()
            except Book.DoesNotExist, BookInstance.DoesNotExist:
                return HttpResponse(u'Não conseguimos cadastrar o livro')
            else:
                bform = BookForm()       
                msg = 's'
    else:
        bform = BookForm()
    return render_to_response('bookLoan/register_book.html', {'bform': bform, 'msg': msg}, context_instance=RequestContext(request))

# get all the books that user borrowed to
# friends and that friends borrowed to user
@login_required
@csrf_exempt
def loans_profile(request):

    me = request.user.get_profile()
    
    with_me = BookLoan.objects.filter(requester=me)
    with_others = BookLoan.objects.filter(book__owner=me)
    
    return render_to_response("bookLoan/profile_loans.html",
                              {'with_me': with_me, 'with_others': with_others},
                              RequestContext(request))

#get a loan id and show its status
@login_required    
def loan_item(request, loan_id):
    my_profile = request.user.get_profile()
    
    loan_item = get_object_or_404(BookLoan, pk=loan_id)
    
    if (loan_item.requester == my_profile or loan_item.book.owner == my_profile) and loan_item.status != BookLoan.RETURNED:
        until = loan_item.date + timedelta(int(loan_item.ndays))
        return render_to_response("bookLoan/loan_detail.html",
                              {'loan':loan_item, 'until':until}, context_instance=RequestContext(request))       
    else:
        return HttpResponse('<h1>Voce nao participa desse emprestimo! Saia daqui!</h1>')


def profile(request, username, tag="all"):
         
    try:
        reader = User.objects.get(username=username).get_profile()
    except Reader.DoesNotExist:
        raise Http404
    
    user = request.user
    
    # if I am authenticated and acessing my own profile
    if user.is_authenticated() and user.username == username:
                    
        if user.is_active:
            reader = user.get_profile()
            
            # paginate his/her friends books
            book_list = reader.get_my_lib()
            book_paginator = Paginator(book_list, 15)
            if 'p' in request.GET:
                try:
                    page = int(request.GET.get('p'))
                except ValueError:
                    page = 1
            else:
                page = 1
            try:
                books = book_paginator.page(page)
            except PageNotAnInteger:
                # if page is not an integer, deliver first page.
                books = book_paginator.page(1)
            except EmptyPage:
                # if page is out of range, deliver last page of results.
                books = book_paginator.page(book_paginator.num_pages)
    
            return render_to_response("bookLoan/my_profile.html",
                                      {'books': books},
                                      RequestContext(request))
        else:
            return HttpResponse('<h1>Usuário cadastrado, mas inativo</h1>')
            
            
    tags = Tag.objects.usage_for_model(BookInstance, counts=True, filters=dict(owner__in=[reader.id]))
    calculate_cloud(tags, steps=6, distribution=dist_LOG)
    is_friend = False
    if user.is_authenticated():
        is_friend = user.get_profile().is_friend(reader)
    return render_to_response("bookLoan/other_profile.html",
                              {'reader':reader,
                               'is_friend': is_friend, 
                               'tags': tags}, 
                              RequestContext(request))
    

# get an book id and, if logged user
# already has an instance of that,
# add another. If don't, create one.
# If something goes wrong, send
# an error message.
@login_required
def ajax_add_book(request, book_id):
    if request.is_ajax():
        try:
            book = Book.objects.get(pk=book_id)
        except Book.DoesNotExist:
            result = u'0'
        else:
            try:
                instance = BookInstance.objects.get(owner=request.user.get_profile(), book=book)
                instance.quantity += 1
            except BookInstance.DoesNotExist:
                instance = BookInstance()
                instance.book = book
                instance.owner = request.user.get_profile()
            instance.save()
            result = str(instance.quantity)
    else:
        result = u'0'
    json = simplejson.dumps(result)
    return HttpResponse(json, mimetype='application/json')

# Takes an book instance id and, if
# the logged user is an owner's friend and 
# instance isn't already borrowed and it
# exists, create and sends a book request
# to owner' email. If owner and logged
# reader are the same, do not do anything
@login_required
def book_request(request, instance_id):
    
    # verify if instance exists
    try:
        instance = BookInstance.objects.get(pk=instance_id)
    except BookInstance.DoesNotExist:
        raise Http404
    
    # verify friendship
    requester = request.user.get_profile()
    if not requester.is_friend(instance.owner):
        return HttpResponseForbidden(u'Permissão negada. É necessário ser\
            amigo do dono do livro para solicitar empréstimo.')
        
    if instance.is_borrowed():
        return HttpResponseForbidden(u'Permissão negada. O livro\
            solicitado já está emprestado')
        
    
    if request.method == 'POST':
        brform = RequestBookForm(request.POST)
        if brform.is_valid():
            book_loan = brform.save()
            #send notification email to owner
            try:
                send_book_request_email(request, book_loan)
            except SMTPException:
                return render_to_response('error.html')
            
            return HttpResponseRedirect(requester.get_absolute_url())
    else:
        brform = RequestBookForm(initial={'book': instance, 'requester': requester})
        
    instance = BookInstance.objects.get(pk=instance_id)
    return render_to_response('bookLoan/book_request.html',{'brform': brform, 'instance': instance},
                              context_instance=RequestContext(request))

# Takes an book loan id with waiting
# status and if the logged user is
# the book owner, gives an response
# to requester.
@login_required
def book_response(request, book_loan_id):
    
    # verify if book request exists
    try:
        book_loan = BookLoan.objects.get(pk=book_loan_id)
    except BookLoan.DoesNotExist:
        raise Http404
    
    # Maybe loan is already answered. Redirect to its details page
    if not book_loan.status == BookLoan.WAITING:
        return HttpResponseRedirect(reverse('loan_detail', kwargs={'loan_id': book_loan_id}))
    
    # if logged user is not the owner,
    # logout him and redirect to login
    owner = request.user.get_profile()
    if not owner.owns(book_loan.book):
        logout(request)
        return HttpResponseRedirect(reverse('login') + '?next=' + request.path)
    
    if request.method == 'POST':
        updated_loan = book_loan
        #TODO: sanitize this user input
        updated_loan.text_reply = request.POST['text_reply']
        if request.POST['answer'] == 'Sim':
            updated_loan.status = BookLoan.ACTIVE
            updated_loan.date = date.today()
        else:
            updated_loan.status = BookLoan.DENIED
        
        # send email to requester
        try:
            send_book_response_email(request, book_loan)
        except SMTPException:
            return render_to_response('error.html')
        
        updated_loan.save()
        return HttpResponseRedirect(owner.get_absolute_url())
    return render_to_response('bookLoan/book_response.html', {'book_loan': book_loan},
                              RequestContext(request))
    
@login_required
def ajax_book_returning_confirmation(request, book_loan_id, resp):
    
    if resp != 'n' and resp != 's':
        return HttpResponseBadRequest(u'Caminho inválido!')
    
    if request.is_ajax():
        try:
            # verify if book_loan request exists
            book_loan = BookLoan.objects.get(pk=book_loan_id)
            
            # verify if logged user is the owner,
            owner = request.user.get_profile()
            if not owner.owns(book_loan.book):
                raise Exception
            
            # Maybe loan is already returned. 
            if book_loan.status != BookLoan.RETURNING:
                raise Exception
            # TODO: send notification email
            if str(resp) == 's':
                book_loan.status = BookLoan.RETURNED
            else:
                book_loan.status = BookLoan.ACTIVE
            book_loan.save()       
            result = u'1'
            
        except Exception:
            result = u'0'
            json = simplejson.dumps(result)
            return HttpResponse(json, mimetype='application/json')
       
    json = simplejson.dumps(result)
    return HttpResponse(json, mimetype='application/json')

@login_required
def ajax_book_returning(request, book_loan_id):
    
    if request.is_ajax():
        # verify if book_loan request exists
        try:
            book_loan = BookLoan.objects.get(pk=book_loan_id)
        except BookLoan.DoesNotExist:
            raise Http404
       
        # Maybe loan is already returned. 
        if book_loan.status != BookLoan.ACTIVE:
            return HttpResponseForbidden(u'Permissão negada. O livro\
                não esta mais emprestado')
    
        # if logged user is not the requester,
        # logout him and redirect to login
        requester = request.user.get_profile()
        if not requester == book_loan.requester:
            logout(request)
            return HttpResponseRedirect(reverse('login') + '?next=' + request.path)
                
        #send notification email to owner
        try:
            send_book_returning_email(request, book_loan)
        except SMTPException:
            return render_to_response('error.html')
     
        book_loan.status = BookLoan.RETURNING
        book_loan.save()       
        result = str('1')
    else:
        result = u'0'
    json = simplejson.dumps(result)
    return HttpResponse(json, mimetype='application/json')


def autocomplete(request, limit=5):
    # default return list
    results = []
    if request.method == "GET":
        if request.GET.has_key(u'o_que') and request.GET.has_key(u'term'):
            value = request.GET[u'term']
            what = request.GET[u'o_que']
            # ignore queries shorter than length 3
            if len(value) > 2:
                if what == 'editora':
                    model_results = Publisher.objects.filter(name__icontains=value)[:limit]
                elif what == 'autor':
                    model_results = Author.objects.filter(name__icontains=value)[:limit]
                results = [ x.name for x in model_results ]
    json = simplejson.dumps(results)
    return HttpResponse(json, mimetype='application/json')

@login_required
def instance(request, instance_id):
    try:
        instance = BookInstance.objects.get(pk=instance_id)
    except BookInstance.DoesNotExist:
        raise Http404()

    owner = instance.owner
    me = request.user.get_profile()
    i_am_owner = False
    friend_of_owner = False
    
    if me == owner:
        i_am_owner = True
    if me.is_friend(owner):
        friend_of_owner = True
    
    more_from_friend = []    
    my_friends = me.get_friends()
    for f in my_friends:
        if f != owner:
            ins = f.has_instance_of(instance.book.id)
            if ins:
                more_from_friend.extend([ins])
    
    friends_ratings = instance.get_friends_rates(me)
        
    #give the list of ratings and number per page
    paginator = Paginator(friends_ratings, 5)
    
    try:
        page = paginator.page(int(request.GET.get('page', 1)))
    except InvalidPage:
        raise Http404("Página sem resultados!")
    
    for fff in page.object_list:
        print "page--" + fff.evaluator.user.get_full_name(), fff.comments
    
    return render_to_response("bookLoan/instance.html",
                              {"instance": instance,
                               "owner": i_am_owner,
                               "friend": friend_of_owner,
                               "page": page,
                               "paginator": paginator,
                               "more_from_friend": more_from_friend},
                              RequestContext(request))
@login_required
def edit_instance(request, instance_id):
    try:
        instance = BookInstance.objects.get(pk=instance_id)
    except BookInstance.DoesNotExist:
        raise Http404()
    
    if request.user.get_profile() != instance.owner:
        return HttpResponseForbidden(u'Permissão negada. Você não é o dono do exemplar')
    
    if request.method == 'POST':
        next_url=reverse('instance', kwargs={'instance_id':instance.id})
        edit_form = EditBookInstanceForm(request.POST,request.FILES,instance=instance)
        if edit_form.is_valid():  
            edit_form.save()
            #perfil alterado salvo
            return HttpResponseRedirect(next_url)
    else:
        edit_form = EditBookInstanceForm(instance=instance)            
    return render_to_response('bookLoan/edit_instance.html', {'form': edit_form,
                                                              'instance':instance},
                              context_instance=RequestContext(request))

def all_my_books(request):
    return HttpResponse("Ainda não estou implementada")
