# -*- coding: utf-8 -*-
from django.forms.models import ModelForm
from django import forms
from email.utils import getaddresses, formataddr
from django.utils.translation import ugettext_lazy as _

from bookLoan.models import FriendRequest


class EmailListField(forms.CharField):
    default_error_messages = {
        'invalid': _('Por favor, entre com uma lista de emails válidos.')
    }

    def clean(self, value):
        value = super(EmailListField, self).clean(value)

        field = forms.EmailField()

        try:
            return [
                formataddr((name, field.clean(addr)))
            for name, addr in getaddresses([value])]
        except forms.ValidationError:
            raise forms.ValidationError(self.error_messages['invalid'])


class FriendRequestForm(ModelForm):
    
    class Meta:
        model = FriendRequest
        fields = ('text','requester','friend')
        widgets = {
                   'text': forms.Textarea(),
                   'requester': forms.HiddenInput(),
                   'friend': forms.HiddenInput(),
                   }

class InviteFriendForm(forms.Form):
    to = EmailListField(label="Email(s)", required=True,
                        help_text="Separe os emails por vírgula para convidar \
                        mais amigos")
    message = forms.CharField(label="Mensagem", widget=forms.Textarea(), required=False, help_text="Opcional")
