#-*- coding: utf-8 -*-


from django.shortcuts import render_to_response, get_object_or_404
from django.template.context import RequestContext
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse

from bookLoan.models import FriendRequest
from friendship.forms import FriendRequestForm, InviteFriendForm
from smtplib import SMTPException
from bookLoan.utils import send_invite_email, send_request_friend_email,\
    send_added_friend_email



@login_required
def ask_friendship_request(request, username, template_name):
    myself = request.user.get_profile()
    friend = get_object_or_404(User, username=username).get_profile()
    
    # If we already are friends, go home
    if friend.is_friend(myself):
        return HttpResponseRedirect(myself.get_absolute_url())
            
    try:
        FriendRequest.objects.get(requester=myself, friend=friend)
    except FriendRequest.DoesNotExist:
        if request.method == 'POST':
            # create a brand-new friend request
            requestf_form = FriendRequestForm(data=request.POST)
            if requestf_form.is_valid():  
                requestf_form.save()
                try:
                    send_request_friend_email(friend.user, myself.user, request)
                except SMTPException:
                    render_to_response('error.html',
                                       {'message': "Não foi possível enviar email de notificação para o(a) amigo(a)."},
                                       RequestContext(request))
                return HttpResponseRedirect(myself.get_absolute_url())
        else:
            requestf_form = FriendRequestForm(initial={'requester': myself.id,
                                                       'friend': friend.id})            
        return render_to_response(template_name, {'form': requestf_form, 'friend':friend},
                                  context_instance=RequestContext(request))
    else:
        # There is an active friend request
        return render_to_response('friendship/ask_friendship_redundant.html', {'friend':friend},
                              context_instance=RequestContext(request))

@login_required
def ask_friendship_confirm(request, username, template_name):
    myself = request.user.get_profile()
    friend = get_object_or_404(User, username=username).get_profile()
    
    try:
        req_f = FriendRequest.objects.get(requester=friend, friend=myself)
    except FriendRequest.DoesNotExist: 
        return Http404
    
    if request.method == 'POST':
        req_f.delete()
        confirmation = request.POST['confirmed']
        if confirmation == 'Sim':
            myself.add_friend(friend)
            try:
                send_added_friend_email(friend.user, myself.user, request)
            except SMTPException:
                pass
                    
        return HttpResponseRedirect(myself.get_absolute_url())
    else:    
        return render_to_response(template_name, {'friend':friend, 'req_f': req_f},
                                  RequestContext(request))

@login_required
def invite_a_friend(request):
    if request.method == 'POST':
        iform = InviteFriendForm(request.POST)
        if iform.is_valid():
            custom_msg = iform.cleaned_data['message']
            error_list = []
            for email_addr in iform.cleaned_data['to']:
                try:
                    send_invite_email(request, email_addr, custom_msg)
                except SMTPException:
                    error_list.add(email_addr)
            if error_list:
                return render_to_response('friendship/invite_errors.html', {'error_list': error_list},
                                      RequestContext(request))
            return HttpResponseRedirect(reverse('welcome'))
    else:
        iform = InviteFriendForm()
    return render_to_response('friendship/invite.html', {'form': iform},
                              RequestContext(request))
