#-*- coding: utf-8 -*-
from django.shortcuts import render_to_response
from django.template.context import RequestContext
from django.http import HttpResponseRedirect, Http404,\
    HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from bookLoan.models import BookInstance, BookRating
from ratingSystem.forms import BookCommentWithRatingForm

@login_required
def rate_book(request, instance_id):
    try:
        instance = BookInstance.objects.get(pk=instance_id)
    except BookInstance.DoesNotExist:
        raise Http404()
    
    # verify if the instance_book belong to friends
    requester = request.user.get_profile()
    if instance not in requester.get_my_lib_and_my_books():
        return HttpResponseForbidden(u'Permissão negada. Só são permitidas\
            avalições de livros seus ou de seus amigos.')    
    
    if request.method == 'POST':
        next_url=reverse('instance', kwargs={'instance_id':instance.id})
        rate_form = BookCommentWithRatingForm(request.POST)
        if rate_form.is_valid():
            try:
                rating = BookRating.objects.get(evaluator=requester, rated_book=rate_form.cleaned_data['rated_book'])
            except BookRating.DoesNotExist:
                rating = BookRating(evaluator=requester, rated_book=rate_form.cleaned_data['rated_book'])
            rating.comments=rate_form.cleaned_data['comments']
            rating.rating=rate_form.cleaned_data['rating']
            rating.save()
            return HttpResponseRedirect(next_url)
    else:
        try:
            rating = BookRating.objects.get(evaluator=requester, rated_book=instance.book)
            rate_form = BookCommentWithRatingForm(instance=rating)
        except BookRating.DoesNotExist:
            rate_form = BookCommentWithRatingForm(initial={'evaluator': requester.id,
                                                   'rated_book': instance.book})            
    return render_to_response('ratingSystem/rate_book.html', {'form': rate_form,
                                                              'instance':instance},
                              context_instance=RequestContext(request))
