# Django settings for bigLib project.
import databases
import os

SETTINGS_DIR = os.path.abspath(os.path.dirname(__file__))

ROOT_DIR = os.path.join(SETTINGS_DIR, '../../')

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
     ('Claudivan Ribeiro', 'claudivan.rbr@gmail.com'),
     ('Tiago Nicolosi Bomventi', 'tiagonb@gmail.com'),
)

MANAGERS = ADMINS

FIXTURE_DIRS = (
   # FIXME doesn't load
   os.path.join(SETTINGS_DIR, 'fixtures/sites/'),
)

DATABASES = databases.DATABASES

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'America/Sao_Paulo'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'pt-BR'

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = os.path.join(SETTINGS_DIR,'../../media/')


# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash.
# Examples: "http://media.lawrence.com/media/", "http://example.com/media/"
MEDIA_URL = '/media/'

# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = os.path.join(SETTINGS_DIR, '../static/')

# URL prefix for static files.
# Example: "http://media.lawrence.com/static/"
STATIC_URL = '/static/'

# URL prefix for admin static files -- CSS, JavaScript and images.
# Make sure to use a trailing slash.
# Examples: "http://foo.com/static/admin/", "/static/admin/".
ADMIN_MEDIA_PREFIX = '/static/admin/'

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = '!fdgt(t!ut=1^vbx80febc*1zh3$m$i!5-l8ej*b1sudb*9use'

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
#     'django.template.loaders.eggs.Loader',
)

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

ROOT_URLCONF = 'bigLib.urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os.path.join(SETTINGS_DIR,'../../templates'),
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    'tagging',
    'haystack',
    'bookLoan',
    'accounts',
    'friendship',
    # Uncomment the next line to enable admin documentation:
    # 'django.contrib.admindocs',
)

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

# Do authentication
AUTHENTICATION_BACKENDS = (
    'accounts.backends.EmailModelBackend',
    'django.contrib.auth.backends.ModelBackend',
)


AUTH_PROFILE_MODULE = 'bookLoan.Reader'

#TODO: change login url to named url

LOGIN_URL= '/login/'
LOGOUT_URL='/logout/'
LOGIN_REDIRECT_URL='/logged/'

#configuracoes do tagging
FORCE_LOWERCASE_TAGS=True
MAX_TAG_LENGTH=30

#configuracoes do registration
EMAIL_HOST = 'smtps.bol.com.br'
EMAIL_HOST_USER = 'biglib.sa@bol.com.br'
EMAIL_HOST_PASSWORD = 'libbig32'
EMAIL_PORT = 587
EMAIL_SUBJECT_PREFIX = '[bigLib] - '
#EMAIL_USE_TLS = True
#DEFAULT_FROM_EMAIL = 'nao.responda@biglib.com.br'
DEFAULT_FROM_EMAIL = EMAIL_HOST_USER

#haystack config variable
HAYSTACK_SITECONF = 'bigLib.search_sites'
HAYSTACK_SEARCH_ENGINE = 'whoosh'
HAYSTACK_WHOOSH_PATH = os.path.join(ROOT_DIR,'index_haystack')


ACCOUNT_ACTIVATION_DAYS = 7
EMAIL_CHANGE_ACTIVATION_DAYS = 3


